import { BrowserRouter as Router } from 'react-router-dom';
import { act } from "react-dom/test-utils";
import { render, cleanup, fireEvent, screen } from '@testing-library/react';
import '@testing-library/jest-dom';
import mockConsole from "jest-mock-console";
import _ from 'lodash';

import { ProjectCreate } from './create';
import ProjectService from '../../services/project.service';
import CycleService from '../../services/cycle.service';

import ProjectServiceMock from '../../__mocks__/project.service.data';

let projectCategoriesSpy, allCycleSpy, periodCategoriesSpy, saveProjectSpy, resourcesSpy, projectResourceDefaultsSpy, fileSystemSpy, clusterSpy;
let restoreConsole;

beforeEach(() => {
    restoreConsole = mockConsole();
    setMockSpy();
});

afterEach(() => {
    restoreConsole();
    // cleanup on exiting
    clearMockSpy();
    cleanup();
});

/**
 * To set mock spy for Services that have API calls to the back end to fetch data
 */
const setMockSpy = (() => {
    projectCategoriesSpy = jest.spyOn(ProjectService, 'getProjectCategories');
    projectCategoriesSpy.mockImplementation(() => { return Promise.resolve(ProjectServiceMock.project_categories) });
    periodCategoriesSpy = jest.spyOn(ProjectService, 'getPeriodCategories');
    periodCategoriesSpy.mockImplementation(() => { return Promise.resolve(ProjectServiceMock.period_categories) });
    allCycleSpy = jest.spyOn(CycleService, 'getAllCycles');
    allCycleSpy.mockImplementation(() => {
        return Promise.resolve([{ url: "http://localhost:3000/api/cycle/Cycle-0", name: 'Cycle-0' },
        { url: "http://localhost:3000/api/cycle/Cycle-1", name: 'Cycle-1' }]);
    });
    resourcesSpy = jest.spyOn(ProjectService, 'getResources');
    resourcesSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.resources);
    });
    projectResourceDefaultsSpy = jest.spyOn(ProjectService, 'getDefaultProjectResources');
    projectResourceDefaultsSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.projectResourceDefaults);
    });

    saveProjectSpy = jest.spyOn(ProjectService, 'saveProject');
    saveProjectSpy.mockImplementation((project, projectQuota) => {
        expect(projectQuota);
        project.url = `http://localhost:3000/api/project/${project.name}`;
        return Promise.resolve(project)
    });
    fileSystemSpy = jest.spyOn(ProjectService, 'getFileSystem');
    fileSystemSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.fileSystem)
    })
    clusterSpy = jest.spyOn(ProjectService, 'getCluster');
    clusterSpy.mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.cluster)
    })
});

const clearMockSpy = (() => {
    projectCategoriesSpy.mockRestore();
    periodCategoriesSpy.mockRestore();
    allCycleSpy.mockRestore();
    saveProjectSpy.mockRestore();
    fileSystemSpy.mockRestore();
    clusterSpy.mockRestore();
});

it("renders without crashing with all back-end data loaded", async () => {
    console.log("renders without crashing with all back-end data loaded ------------------------");

    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });
    const spinButtons = content.queryAllByRole("spinbutton");
    expect(content.queryByText('Project - Add')).not.toBe(null);        // Page loaded successfully
    expect(projectCategoriesSpy).toHaveBeenCalled();                    // Mock Spy called successfully
    fireEvent.click(screen.getAllByText("Select Project Category")[0].parentElement.parentElement.parentElement.children[3]);
    expect(content.queryByText('Regular')).toBeInTheDocument();         // Project Category Dropdown  loaded successfully
    fireEvent.click(screen.getAllByText("Select Period Category")[0].parentElement.parentElement.parentElement.children[3]);
    expect(content.queryByText('Single Cycle')).toBeInTheDocument();    // Period Category Dropdown loaded successfully
    fireEvent.click(screen.getAllByRole("listbox")[0].parentElement.parentElement);
    expect(content.queryByText('Cycle-0')).toBeInTheDocument();         // Cycle multi-select loaded successfully
    expect(content.queryAllByText('Add Resources').length).toBe(2);     // Resource Dropdown loaded successfully
    fireEvent.click(screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]);
    expect(content.queryByText('LOFAR Observing Time Commissioning')).toBeInTheDocument();         // Resources other than Default Resources listed in dropdown
    expect(_.filter(spinButtons, { "id": "LOFAR Observing Time Commissioning" }).length).toBe(0);       // No resources other than Default Resources listed to get input
    expect(_.filter(spinButtons, { "id": "LOFAR Observing Time" })[0].value).toBe("1");         // Default Resource Listed with default value
});

it("Save button disabled initially when no data entered", async () => {
    console.log("Save button disabled initially when no data entered -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});

it("Save button enabled when mandatory data entered", async () => {
    console.log("Save button enabled when mandatory data entered -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    const nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const spinButtons = content.queryAllByRole("spinbutton");
    const rankInput = spinButtons.filter(function (element) { return element.id === "proj-rank" })[0];
    fireEvent.change(nameInput, { target: { value: 'OSR' } });
    expect(nameInput.value).toBe("OSR");

    fireEvent.change(descInput, { target: { value: 'OSR' } });
    expect(descInput.value).toBe("OSR");

    fireEvent.change(rankInput, { target: { value: 1 } });
    expect(rankInput.value).toBe("1");

    await new Promise((r) => setTimeout(r, 1000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
});


it("renders Save button enabled when all data entered", async () => {
    console.log("renders Save button enabled when all data entered -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    const nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const spinButtons = content.queryAllByRole("spinbutton");
    const rankInput = spinButtons.filter(function (element) { return element.id === "proj-rank" })[0];
    const trigPrioInput = spinButtons.filter(function (element) { return element.id === "trig_prio" })[0];
    const trigger = content.getByLabelText(/trigger/i);

    fireEvent.change(nameInput, { target: { value: 'OSR' } });
    expect(nameInput.value).toBe("OSR");

    fireEvent.change(descInput, { target: { value: 'OSR' } });
    expect(descInput.value).toBe("OSR");

    fireEvent.change(rankInput, { target: { value: 1 } });
    expect(rankInput.value).toBe("1");

    expect(trigPrioInput.value).toBe("1000");                       // Check for default value
    fireEvent.change(trigPrioInput, { target: { value: 100 } });
    expect(trigPrioInput.value).toBe("100");                        // Check for new value

    fireEvent.click(trigger);
    expect(trigger.hasAttribute("checked")).toBeTruthy();

    // Before selecting Project Category
    expect(content.queryAllByText('Select Project Category').length).toBe(2);
    expect(content.queryAllByText('Regular').length).toBe(0);
    // expect(content.getAllByRole("listbox")[0].children.length).toBe(2);
    // After selecting Project Category
    fireEvent.click(screen.getAllByText("Select Project Category")[0].parentElement.parentElement.parentElement.children[3]);
    const projCatInput = screen.getAllByText("Regular")[0];
    fireEvent.click(projCatInput);
    expect(content.queryAllByText('Select Project Category').length).toBe(0);

    expect(content.queryAllByText('Regular').length).toBe(3);

    // Before selecting Period Category
    expect(content.queryAllByText('Select Period Category').length).toBe(2);
    expect(content.queryAllByText('Single Cycle').length).toBe(0);
    // expect(content.getAllByRole("listbox")[1].children.length).toBe(2);
    // After selecting Period Category
    fireEvent.click(screen.getAllByText("Select Period Category")[0].parentElement.parentElement.parentElement.children[3]);
    const projPeriodInput = screen.getAllByText("Single Cycle")[0];
    fireEvent.click(projPeriodInput);
    expect(content.queryAllByText('Select Period Category').length).toBe(0);
    expect(content.queryAllByText('Single Cycle').length).toBe(3);

    // Before selecting Cycle
    expect(content.queryAllByText('Cycle-0').length).toBe(0);
    // expect(content.getAllByRole("listbox")[2].children.length).toBe(2);
    // After selecting Cycle
    fireEvent.click(screen.getAllByRole("listbox")[0].parentElement.parentElement);
    const cycleInput = screen.getAllByText("Cycle-0")[0];
    fireEvent.click(cycleInput);
    expect(content.queryAllByText('Cycle-0').length).toBe(2);
    await new Promise((r) => setTimeout(r, 1000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
});

it("save project with mandatory fields", async () => {
    console.log("save project -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);

    const spinButtons = content.queryAllByRole("spinbutton");
    const rankInput = spinButtons.filter(function (element) { return element.id === "proj-rank" })[0];

    fireEvent.change(nameInput, { target: { value: 'OSR' } });
    expect(nameInput.value).toBe("OSR");
    fireEvent.change(descInput, { target: { value: 'OSR' } });
    expect(descInput.value).toBe("OSR");
    fireEvent.change(rankInput, { target: { value: "1" } });
    expect(rankInput.value).toBe("1");
    await new Promise((r) => setTimeout(r, 1000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('projectId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);

    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveProjectSpy).toHaveBeenCalled();
    //After project saved successful
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");
});

it("save project with default resources", async () => {
    console.log("save project with default resources -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);
    const spinButtons = content.queryAllByRole("spinbutton");
    const rankInput = spinButtons.filter(function (element) { return element.id === "proj-rank" })[0];

    fireEvent.change(nameInput, { target: { value: 'OSR' } });
    expect(nameInput.value).toBe("OSR");
    fireEvent.change(descInput, { target: { value: 'OSR' } });
    expect(descInput.value).toBe("OSR");
    fireEvent.change(rankInput, { target: { value: 1 } });
    expect(rankInput.value).toBe('1');
    await new Promise((r) => setTimeout(r, 1000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('projectId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);

    const lofarObsTimeInput = _.filter(spinButtons, { "id": "LOFAR Observing Time" })[0];
    fireEvent.change(lofarObsTimeInput, { target: { value: 10 } });
    expect(lofarObsTimeInput.value).toBe('10');

    const lofarObsTimeAInput = _.filter(spinButtons, { "id": "LOFAR Observing Time prio A" })[0];
    fireEvent.change(lofarObsTimeAInput, { target: { value: 15 } });
    expect(lofarObsTimeAInput.value).toBe('15');

    const lofarObsTimeBInput = _.filter(spinButtons, { "id": "LOFAR Observing Time prio B" })[0];
    fireEvent.change(lofarObsTimeBInput, { target: { value: 20 } });
    expect(lofarObsTimeBInput.value).toBe('20');

    const cepProcTimeInput = _.filter(spinButtons, { "id": "CEP Processing Time" })[0];
    fireEvent.change(cepProcTimeInput, { target: { value: 5 } });
    expect(cepProcTimeInput.value).toBe('5');
    fireEvent.change(cepProcTimeInput, { target: { value: '' } });
    expect(cepProcTimeInput.value).toBe('');

    const ltaStorageInput = _.filter(spinButtons, { "id": "LTA Storage" })[0];
    fireEvent.change(ltaStorageInput, { target: { value: 2 } });
    expect(ltaStorageInput.value).toBe('2');

    const noOfTriggerInput = _.filter(spinButtons, { "id": "Number of triggers" })[0];
    fireEvent.change(noOfTriggerInput, { target: { value: 3 } });
    expect(noOfTriggerInput.value).toBe('3');

    const lofarSupTimeInput = _.filter(spinButtons, { "id": "LOFAR Support Time" })[0];
    fireEvent.change(lofarSupTimeInput, { target: { value: 25 } });
    expect(lofarSupTimeInput.value).toBe('25');

    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveProjectSpy).toHaveBeenCalled();

    //After project saved successful
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");

});

it("save project with added resources", async () => {
    console.log("save project with added resources -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    let nameInput = content.queryByTestId('name');
    const descInput = content.queryByTestId('description');
    const createAnother = content.queryByTestId('createAnother');
    fireEvent.click(createAnother);
    let spinButtons = content.queryAllByRole("spinbutton");
    const rankInput = spinButtons.filter(function (element) { return element.id === "proj-rank" })[0];

    fireEvent.change(nameInput, { target: { value: 'OSR' } });
    expect(nameInput.value).toBe("OSR");
    fireEvent.change(descInput, { target: { value: 'OSR' } });
    expect(descInput.value).toBe("OSR");
    fireEvent.change(rankInput, { target: { value: 1 } });
    expect(rankInput.value).toBe("1");
    await new Promise((r) => setTimeout(r, 2000));
    expect(content.queryByTestId('save-btn').hasAttribute("disabled")).toBeFalsy();
    expect(content.queryByTestId('projectId').value).toBe("");
    expect(content.queryByText("Success")).toBe(null);

    const lofarObsTimeInput = _.filter(spinButtons, { "id": "LOFAR Observing Time" })[0];
    fireEvent.change(lofarObsTimeInput, { target: { value: 10 } });
    expect(lofarObsTimeInput.value).toBe('10');

    const lofarObsTimeAInput = _.filter(spinButtons, { "id": "LOFAR Observing Time prio A" })[0];
    fireEvent.change(lofarObsTimeAInput, { target: { value: 15 } });
    expect(lofarObsTimeAInput.value).toBe('15');

    const lofarObsTimeBInput = _.filter(spinButtons, { "id": "LOFAR Observing Time prio B" })[0];
    fireEvent.change(lofarObsTimeBInput, { target: { value: 20 } });
    expect(lofarObsTimeBInput.value).toBe('20');

    const cepProcTimeInput = _.filter(spinButtons, { "id": "CEP Processing Time" })[0];
    fireEvent.change(cepProcTimeInput, { target: { value: 5 } });
    expect(cepProcTimeInput.value).toBe('5');

    const ltaStorageInput = _.filter(spinButtons, { "id": "LTA Storage" })[0];
    fireEvent.change(ltaStorageInput, { target: { value: 2 } });
    expect(ltaStorageInput.value).toBe('2');

    const noOfTriggerInput = _.filter(spinButtons, { "id": "Number of triggers" })[0];
    fireEvent.change(noOfTriggerInput, { target: { value: 3 } });
    expect(noOfTriggerInput.value).toBe('3');

    const lofarSupTimeInput = _.filter(spinButtons, { "id": "LOFAR Support Time" })[0];
    fireEvent.change(lofarSupTimeInput, { target: { value: 25 } });
    expect(lofarSupTimeInput.value).toBe('25');

    // Before selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    expect(content.queryAllByText('LOFAR Observing Time Commissioning').length).toBe(0);
    expect(_.filter(spinButtons, { "id": "LOFAR Observing Time Commissioning" }).length).toBe(0);
    fireEvent.click(screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]);
    const addResourceInput = screen.getAllByText('LOFAR Observing Time Commissioning')[0];
    fireEvent.click(addResourceInput);
    // After selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(0);
    expect(content.queryAllByText('LOFAR Observing Time Commissioning').length).toBe(3);

    const addResourceBtn = content.queryByTestId('add_res_btn');
    fireEvent.click(addResourceBtn);
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    spinButtons = content.queryAllByRole("spinbutton");

    const newResourceInput = _.filter(spinButtons, { "id": "LOFAR Observing Time Commissioning" })[0];
    expect(newResourceInput).not.toBe(null);
    fireEvent.change(newResourceInput, { target: { value: 30 } });
    expect(newResourceInput.value).toBe('30');

    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 1000));
    });
    console.log("Checking service calls..");
    expect(saveProjectSpy).toHaveBeenCalled();

    //After project saved successful
    nameInput = content.queryByTestId('name');
    expect(nameInput.value).toBe("");
    expect(content.queryByTestId('save-btn')).toHaveAttribute("disabled");

});



it("remove default resource and added resource", async () => {
    console.log("remove default resource and added resource -----------------------");
    let content;
    await act(async () => {
        content = render(<Router><ProjectCreate location={{ pathname: "/project/create" }} /></Router>);
    });

    // Before selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(2);
    expect(content.queryAllByText('LOFAR Observing Time Commissioning').length).toBe(0);
    expect(content.queryByPlaceholderText('LOFAR Observing Time Commissioning')).toBe(null);
    await act(async () => {
        fireEvent.click(screen.getAllByText("Add Resources")[0].parentElement.parentElement.parentElement.children[3]);
    });
    
    const addResourceInput = screen.getAllByText("LOFAR Observing Time Commissioning")[0];
    fireEvent.click(addResourceInput);
    // After selecting New Resource
    expect(content.queryAllByText('Add Resources').length).toBe(0);
    expect(content.queryAllByText('LOFAR Observing Time Commissioning').length).toBe(3);

    const addResourceBtn = content.queryByTestId('add_res_btn');
    fireEvent.click(addResourceBtn);
    expect(content.queryAllByText('Add Resources').length).toBe(2);

    const spinButtons = content.queryAllByRole("spinbutton");

    expect(_.filter(spinButtons, { "id": "LOFAR Observing Time Commissioning" })[0]).not.toBe(null);

    expect(_.filter(spinButtons, { "id": "CEP Processing Time" })[0]).not.toBe(null);
    expect(content.queryByTestId('CEP Processing Time-btn')).not.toBe(null);

    const removeDefResBtn = content.queryByTestId('CEP Processing Time-btn');
    await act(async () => {
        fireEvent.click(removeDefResBtn);
    });
    expect(_.filter(spinButtons, { "id": "CEP Processing Time" }).length).toBe(0);
    expect(content.queryByTestId('CEP Processing Time-btn')).toBe(null);

    const removeResourceBtn = content.queryByTestId('LOFAR Observing Time Commissioning-btn');
    await act(async () => {
        fireEvent.click(removeResourceBtn);
    });
    expect(content.queryAllByText('Add Resources').length).toBe(2);
 
    expect(content.queryAllByText('LOFAR Observing Time Commissioning').length).toBe(0);
});
