import React, { Component } from 'react';
import { Link } from 'react-router-dom';
import moment from 'moment';
import _ from 'lodash';
import Papa from "papaparse";
import jsPDF from 'jspdf';
import html2canvas from 'html2canvas';
import ReactToPrint from "react-to-print";

import { AutoComplete } from 'primereact/autocomplete';
import { Calendar } from 'primereact/calendar';
import { Checkbox } from 'primereact/checkbox';
import { Button } from 'primereact/button';
import { ProgressBar } from 'primereact/progressbar';
import { AppLoader } from '../../../layout/components/AppLoader';
import { appGrowl } from '../../../layout/components/AppGrowl';
import { CustomDialog } from '../../../layout/components/CustomDialog';
import CycleService from '../../../services/cycle.service';
import ReportService from '../../../services/report.service';
import CycleReportIntro from './report.intro';
import CycleTelescopeTime from './report.telescope.time';
import CycleReportAvgEfficiency from './report.avg.efficiency';
import CycleCompletionLevel from './report.cycle.completion';
import CycleCategoryObservations from './report.category.observations';
import CycleWeeklyEfficiency from './report.weekly.efficiency';
import CycleSiteWiseData from './report.site.data';
import CycleCategoryWiseData from './report.category.data';
import CycleProjectSummary from './report.project.summary';
import CyclesILTLocalUsage from './report.ilt_local.usage';
import CycleFailureRate from './report.failure.rate';
import CycleOverview from './report.overview';

const REPORT_VARIABLE_MAP = {"Intro" : "report_intro",
                             "Overview": "overview",
                             "TelescopeTime" : "telescope_time_distribution",
                             "AverageEfficiency": "average_efficiency",
                             "CompletionLevel": "completion_level",
                             "CategoryObservation": "observation_durations_per_category",
                             "WeeklyEfficiency": "weekly_efficiency",
                             "DataIngestedPerSite": "data_ingested_per_site_and_category",
                             "DataIngestedPerCategory": "data_ingested_per_site_and_category",
                             "ProjectSummary": "projects_summary",
                             "ILT_LocalUsage": "usage_mode",
                             "Failures": "failures"
                            }

/**
 * Main component for Cycle Report
 */
class CycleReportMain extends Component {
    
    constructor(props) {
        super(props);
        this.state = {
            pageLoading: true,
            selectedCycles: [],   // Used by the Autocomplete field
            reportCycles: [],     // Used to generate the report for selected cycles
            reportPeriod: [],        // Period of reporting
            dialog: null
        };
        this.searchCycles = this.searchCycles.bind(this);
        this.selectCycles = this.selectCycles.bind(this);
        this.setReportPeriod = this.setReportPeriod.bind(this);
        this.setReportCycles = this.setReportCycles.bind(this);
        this.setExportData = this.setExportData.bind(this);
        this.downloadCSV = this.downloadCSV.bind(this);
        this.downloadPDF = this.downloadPDF.bind(this);
        this.clearAll = this.clearAll.bind(this);
        this.toggleScroll = this.toggleScroll.bind(this);
        this.csvDialogContent = this.csvDialogContent.bind(this);
        this.showCSVReportDialog = this.showCSVReportDialog.bind(this);
        this.changeCSVOption = this.changeCSVOption.bind(this);
        this.selectAllCSVOptions = this.selectAllCSVOptions.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
    }

    componentDidMount() {
        CycleService.getResources()
        .then(resourceList => {this.resourceList = resourceList});
        CycleService.getAllCyclesWithQuota()
        .then(cycles => {this.cycles = cycles; this.setState({cycles: cycles, pageLoading: false})});
        window.addEventListener('scroll', this.toggleScroll);
    }

    /**
     * Function passed to AutoComplete component to search and list
     * @param {*} event 
     */
    searchCycles(event) {
        setTimeout(() => {
            let suggestedCycles;
            if (!event.query.trim().length) {
                suggestedCycles = [...this.state.cycles];
            }
            else {
                suggestedCycles = this.state.cycles.filter((cycle) => {
                    // For matching the start of the cycle with the search text
                    // return cycle.name.toLowerCase().startsWith(event.query.toLowerCase());
                    // For values containing the search text
                    return cycle.name.toLowerCase().indexOf(event.query.toLowerCase())>=0;
                });
            }
            this.setState({ suggestedCycles });
        }, 250);
    }

    /**
     * Function to set the selected cycles from the AutoComplete component
     * @param {Array} cycles 
     */
    selectCycles(cycles) {
        let reportPeriod = [];
        if (cycles.length > 0) {
            let cycleDates = [];
            // Get all cycle dates
            for (const cycle of cycles) {
                cycleDates.push(moment(cycle.start));
                cycleDates.push(moment(cycle.stop));
            }
            // Get the minimum and maximum date from the cycle dates and set as the report period
            let minCycleDate = _.min(cycleDates);
            let maxCycleDate = _.max(cycleDates);
            if (minCycleDate && maxCycleDate) {
                reportPeriod = [minCycleDate.toDate(), maxCycleDate.toDate()];
            }
        }
        this.setState({selectedCycles: cycles, reportPeriod: reportPeriod});
    }

    /**
     * Set Report period and close the calendar if both values in the range are selected.
     * @param {Array} value - Array of date objects
     */
    setReportPeriod(value) {
        this.setState({reportPeriod: value?value:[]});
        if (value && value.length===2 && value[1]) {
            document.getElementsByClassName('p-datepicker')[0].style.display = "none";
        }
    }
    
    /**
     * Function to generate the PDF of the report.
     */
    async downloadPDF() {
        this.setState({isDownloading: true});
        let reportDivs = document.getElementsByClassName('report-div');
        // Create a PDF document with landscape orientation, 1st report div width and the max height of all report divs in pixels
        const pdf = new jsPDF('l', 'px',  [ (reportDivs[0].clientWidth+100), (reportDivs[0].clientHeight+100)]);
        let pageIndex = 0;
        // Draw each report in canvas and create image of the canvas before saving the pdf
        for (const reportName of _.keys(REPORT_VARIABLE_MAP)) {
            // Create page for each project
            if (pageIndex > 0) {
                pdf.addPage([reportDivs[0].clientWidth+100, reportDivs[pageIndex].clientHeight+100], pageIndex===6?'p':'l');
            }
            const reportElement = document.getElementById(`${reportName}`);
            const options = {
                foreignObjectRendering: true,
                width: reportElement.clientWidth,
                height: reportElement.clientHeight
            };
            const projectCanvas = await html2canvas(reportElement, options);
            pdf.addImage(projectCanvas.toDataURL('image/jpeg'), 'JPEG', 50, 50, reportDivs[pageIndex].clientWidth, reportDivs[pageIndex].clientHeight);
            pageIndex++;
        }
        // To open the generated PDF in new window
        // pdf.output('dataurlnewwindow');
        const reportPeriod = `${moment(this.state.reportPeriod[0]).format("DDMMMYYYY")}-${moment(this.state.reportPeriod[1]).format("DDMMMYYYY")}`;
        pdf.save(`Cycle_${reportPeriod}.pdf`);
        this.setState({isDownloading: false});
    }

    /**
     * Report list shown in the dialog as checkbox to include or exclude while exporting CSV files.
     * @returns Component
     */
    csvDialogContent() {
        let allReports = _.keys(REPORT_VARIABLE_MAP);
        allReports = allReports.splice(1);
        const repWithData = _.keys(this.cyclesReportData); //this.state.reportNames;
        let csvOptions = [];
        for (const report of allReports) {
            let csvOption = {name: report};
            csvOption.disabled = repWithData.indexOf(report) < 0;
            csvOption.checked = this.state.reportNames.indexOf(report) >= 0;
            csvOptions.push(csvOption);
        }
        return (
            <div className="fluid csv-options-dlg-content">
                <div className="col-lg-12" style={{padding: '5px'}}>
                    <Checkbox inputId={`cb-all`} value="all" 
                        checked={this.state.reportNames.length === _.keys(REPORT_VARIABLE_MAP).length-1}
                        onChange={() => {this.selectAllCSVOptions()}}></Checkbox>
                    <label htmlFor={`cb-all`} className="p-checkbox-label checkbox-label">All</label>
                </div>
                {csvOptions.map((item, index) => (
                    <div className="col-lg-12" style={{padding: '5px'}} key={item.name}>
                        <Checkbox inputId={`cb-${index}`} value={item.name} 
                            checked={item.checked} disabled={item.disabled}
                            onChange={()=> {this.changeCSVOption(item.name, !item.checked)}}></Checkbox>
                        <label htmlFor={`cb-${index}`} className={`p-checkbox-label checkbox-label${item.disabled?'-disabled':''}`}>{item.name}</label>
                    </div>
                ))}
            </div>);
    }

    /**
     * Function called when report is selected or unselected from the list
     * @param {string} reportName - name of the report passed from the checkbox component of the list dialog
     * @param {boolean} selected - to include or exclude
     */
    changeCSVOption(reportName, selected) {
        let reportNames = this.state.reportNames;
        if (selected) {
            reportNames.push(reportName);
        }   else {
            reportNames = _.filter(reportNames, rep => rep !== reportName);
        }
        this.setState({reportNames: reportNames});
    }

    /**
     * Select or Unselect all reports in the dialog
     */
    selectAllCSVOptions() {
        let repWithData = _.keys(this.cyclesReportData);
        if (repWithData.length > this.state.reportNames.length) {
            this.setState({reportNames: repWithData});
        }   else {
            this.setState({reportNames: []});
        }
    }

    /**
     * To show the dialog with reports to be selected or unselected for downloading as CSV files.
     * @returns 
     */
    showCSVReportDialog() {
        const reportNames = (this.state.reportNames && this.state.reportNames.length > 0)?
                                this.state.reportNames: _.keys(this.cyclesReportData);
        if (reportNames.length === 0) {
            appGrowl.show({severity: 'info', summary: 'No Data', detail: 'None of the report has data to download'});
            return;
        }
        let dialog = { header: "CSV Report(s) to download", width: "40vw", actions: null, showIcon: false};
        dialog.detail = "Following selected reports will be downloaded. Report(s) with no data are disabled. Select/Unselect the reports of your interest and click 'Ok'";
        dialog.content = this.csvDialogContent;
        dialog.onSubmit = this.downloadCSV;
        dialog.actions = [{ id: 'yes', title: 'Ok', callback: this.downloadCSV},
                            { id: 'no', title: 'Cancel', callback: this.closeDialog}]
        this.setState({csvDlgVisible: true, dialog: dialog, reportNames: reportNames});
    }

    /**
     * Closes the dialog.
     */
    closeDialog() {
        this.setState({csvDlgVisible: false});
    }

    /**
     * Function to download the report data in CSV format
     */
    downloadCSV() {
        const reportNames = this.state.reportNames;
        if (reportNames.length === 0) {
            appGrowl.show({severity: 'info', summary: 'No Report Selected', detail: 'No report selected to download'});
        }
        // For every sub-report of the main report get report data
        for(const reportName of reportNames) {
            let csvConfig = {};
            let csvList= [];            //Data list to export
            let colHeaders = [];        //Column header to export. Both no of headers and data values should be same.
            let isColHeaderSet = false; //Flag to identify if colHeader is added
            // Get the whole report data set from MainReport component
            const subReportData = this.cyclesReportData[reportName];
            subReportData.map((data ) => {
                let csvData = {};
                const colKeys = _.keys(data);
                // For every column of the data, replace the column name with the column title
                for (const colKey of colKeys) {
                    let colHeader = _.upperFirst(colKey);
                    if (!isColHeaderSet) {
                        colHeaders.push(colHeader);
                    }
                    csvData[colHeader] = data[colKey];
                }
                csvList.push(csvData);
                isColHeaderSet = true;
                return data;
            });
            // Pass column headers to CSV parser
            if (colHeaders.length > 0) {
                csvConfig.columns = colHeaders;
            }
            // Parse the CSV list and header opject to create CSV string
            const csvString = Papa.unparse(csvList, csvConfig);
            // Create a CSV BLOB from the csvString
            const blob = new Blob([csvString], { type: "text/csv" });

            // Add dummy link to click and download the created CSV
            var a = document.createElement("a");
            document.body.appendChild(a);
            a.style = "display: none";
            var url = window.URL.createObjectURL(blob);
            a.href = url;
            const reportPeriod = `${moment(this.state.reportPeriod[0]).format("DDMMMYYYY")}-${moment(this.state.reportPeriod[1]).format("DDMMMYYYY")}`;
            a.download = `Cycle_${reportName}_${reportPeriod}.csv`;
            a.click();
            window.URL.revokeObjectURL(url);   
        }
        this.setState({csvDlgVisible: false});
    }

    /**
     * Function called to set report cycles from the selected cycles.
     */
    async setReportCycles() {
        this.cyclesReportData = null;
        this.setState({isLoading: true, cyclesReportData: [], reportNames: null});
        const cycleNames = _.map(_.orderBy(this.state.selectedCycles, 'start'), 'name');
        
        let cyclesReportData = [];
        cyclesReportData = _.values(await ReportService.getCyclesReport(cycleNames));
        cyclesReportData.map(cycleReportData => {
            cycleReportData["overview"] = cycleReportData.projects_summary;
            return cycleReportData;
        });
        this.setState({ reportCycles: _.orderBy(_.cloneDeep(this.state.selectedCycles), 'start'),
                        cyclesReportData: cyclesReportData,
                        isLoading: false});
    }

    /**
     * To extract data required for the sub report component from the cycle report API response data.
     * @param {String} reportVariable - sub report variable from the cycle report API response
     * @returns Object - Extracted object for the respective sub report component
     */
    getReportData(reportVariable) {
        let reportData = _.map(this.state.cyclesReportData, data => {
            let repData = { cycle: data.cycle };
            repData[reportVariable] = data[reportVariable] ;
            if (reportVariable === 'overview') {
                repData['completionLevel'] = data['completion_level'];
            }
            return repData;
        });
        return reportData;
    }

    /**
     * Callback function passed to the CycleReport sub components to get the report data to use in CSV export.
     * @param {String} reportName 
     * @param {Object} reportData - All required data from the CycleReport sub component
     */
    setExportData(reportName, reportData) {
        let cyclesReportData = this.cyclesReportData || {};
        cyclesReportData[reportName] = reportData;
        this.cyclesReportData = cyclesReportData;
    }

    /**
     * Clears the report filters and report divs
     */
    clearAll(){
        this.cyclesReportData = {};
        this.setState({selectedCycles:[], reportPeriod:[], reportCycles:[]});
    }

    /**
     * Callback function to window scroll event listener to set or unset flag to show or hide ScrollToTop button
     */
    toggleScroll() {
        const scrolled = document.documentElement.scrollTop;
        if (scrolled > 300){
            this.setState({scrollTopVisible: true});
        } else if (scrolled <= 300){
            this.setState({scrollTopVisible: false});
        }
    }

    /**
     * Function to scroll to top of the window on clicking ScrollToTop button.
     */
    scrollToTop() {
        window.scrollTo({
            top: 75, 
            behavior: 'smooth'});
    }
    /**
     * 
     * @returns margins for print report
     */
    getPageMargins() {
        return `@page { margin: 138px 72px 138px 72px !important; }`;
      }


    render() {
        return(
            <React.Fragment>
                {this.state.pageLoading &&
                    <AppLoader />}
                {!this.state.pageLoading &&
                <div className="report-toolbar p-grid" style={{marginTop: "10px", paddingBottom: "10px", borderBottom: "1px solid lightgrey"}}>
                    <label htmlFor="cycle" className="col-lg-1 col-md-2 col-sm-12">Cycle </label>
                    <AutoComplete className="col-lg-4 col-md-4 col-sm-12" multiple forceSelection autoFocus autoHighlight
                        tooltip="Type few letters of the cycle and select the cycle suggested" tooltipOptions={{position: "top"}}
                        value={this.state.selectedCycles} suggestions={this.state.suggestedCycles}
                        completeMethod={this.searchCycles} field="name" onChange={(e) => this.selectCycles(e.value)} />
                    <label htmlFor="period" className="col-lg-2 col-md-2 col-sm-12" style={{paddingRight: "0px"}}>For Period </label>
                    <div className="col-lg-3 col-md-3 col-sm-12 report-calendar">
                        <Calendar value={this.state.reportPeriod} selectionMode="range" dateFormat="yy-mm-dd"
                            tooltip="Select the date range for the report"
                            showButtonBar onChange={e => this.setReportPeriod(e.value)}></Calendar>
                    </div>
                    <div className="col-lg-2 col-md-1 col-sm-12">
                        <Button label="" className="p-button-primary" icon="pi pi-check" tooltip="Generate Report"
                                onClick={this.setReportCycles} disabled={this.state.selectedCycles.length===0 || this.state.reportPeriod.length!==2} />
                        <Button label="" className="p-button-primary" icon="fas fa-sync-alt" tooltip="Clear All"
                                onClick={this.clearAll} style={{marginLeft:"10px"}} />
                    </div>
                </div>
                }
                {this.state.isLoading &&
                    <AppLoader />}
                {this.state.isDownloading &&
                <div><ProgressBar mode="indeterminate" style={{ height: '5px' }}/></div> }
                {this.state.reportCycles.length>0 &&
                <>
                    <div ref={(el) => (this.reportDownloadBarRef = el)} >   
                        <Link to={{}} className="report-download-bar" style={{color: "#148048"}} 
                            title="Download Report Data in CSV format. Each report data will be downloaded as separate .csv file. Enable auto downloading multiple files in browser settings." 
                            onClick={this.showCSVReportDialog}>
                            <i className="fas fa-file-csv"></i>
                        </Link> 
                        <Link to={{}} className="report-download-bar" style={{color: "#f20f00"}} title="Download Report as PDF" 
                                onClick={() => { // Scroll to the bottom of the report and back to the top before exporting
                                                    this.reportEndRef.scrollIntoView({behavior: "smooth", block: "end"});
                                                    this.reportDownloadBarRef.scrollIntoView({behavior: "smooth", block: "end"});
                                                    setTimeout(this.downloadPDF, 500)}}>
                                <i className="fas fa-file-pdf"></i>
                        </Link> 
                        <ReactToPrint
                            trigger={() => <i className="pi pi-print report-download-bar print-btn" title="Print Report"></i>}
                            content={() => this.reportPrintRef}
                        />
                    </div>
                    <div ref={(el) => (this.reportPrintRef = el)}>
                    <style>{this.getPageMargins()}</style>
                        <div className="report-div" id="Intro">
                            <CycleReportIntro reportPeriod={this.state.reportPeriod} cycles={this.state.reportCycles} />
                        </div>
                        <div className="report-div" id="Overview">
                            <label>Cycle Overview</label>
                            <CycleOverview id="Overview" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["Overview"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="TelescopeTime">
                            <label>Telescope Time Distribution</label>
                            <CycleTelescopeTime id="TelescopeTime" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["TelescopeTime"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="AverageEfficiency">
                            <label>Average Efficiency</label>
                            <CycleReportAvgEfficiency id="AverageEfficiency" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["AverageEfficiency"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="CompletionLevel">
                            <label>Cycle Completion Level</label>
                            <CycleCompletionLevel id="CompletionLevel" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["CompletionLevel"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="CategoryObservation">
                            <label>Observation Hours per Category</label>
                            <CycleCategoryObservations id="CategoryObservation" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["CategoryObservation"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="WeeklyEfficiency">
                            <label>Weekly Efficiency</label>
                            <CycleWeeklyEfficiency id="WeeklyEfficiency" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["WeeklyEfficiency"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="DataIngestedPerSite">
                            <label>Ingested Data per Site</label>
                            <CycleSiteWiseData id="DataIngestedPerSite" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["DataIngestedPerSite"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="DataIngestedPerCategory">
                            <label>Ingested Data per Category</label>
                            <CycleCategoryWiseData id="DataIngestedPerCategory" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["DataIngestedPerCategory"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="ProjectSummary">
                            <label>Projects Summary Overview</label>
                            <CycleProjectSummary id="ProjectSummary" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["ProjectSummary"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="ILT_LocalUsage">
                            <label>ILT vs Local Mode Usage</label>
                            <CyclesILTLocalUsage id="ILT_LocalUsage" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["ILT_LocalUsage"])}
                                setExportData={this.setExportData} />
                        </div>
                        <div className="report-div" id="Failures">
                            <label>Failure Rate</label>
                            <CycleFailureRate id="Failures" 
                                data={this.getReportData(REPORT_VARIABLE_MAP["Failures"])}
                                setExportData={this.setExportData} />
                        </div>
                    </div>
                    {/* Dummy reference div to scroll down before exporting to PDF so that 
                        all components will be rendered properly before exporting */}
                    <div ref={(el) => {this.reportEndRef = el}}></div>
                    {/* Scroll To Top button */}
                    {this.state.scrollTopVisible &&
                        <Button className="p-button-info p-button-raised float-btn-right-bottom"
                            onClick={this.scrollToTop}
                            icon="pi pi-arrow-up" tooltip="Move to top" 
                            tooltipOptions={{position: 'left'}}></Button>
                    }
                </>
                }
                {this.state.csvDlgVisible && 
                    <CustomDialog type="confirmation" visible={this.state.csvDlgVisible}
                        header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                        content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                        onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit}/>
                }
            </React.Fragment>
        );
    }

}

export default CycleReportMain;