import React, { useState, useEffect } from 'react';
import moment from 'moment';
import _ from 'lodash';
import Jeditor from '../../components/JSONEditor/JEditor';
import UnitConversion from '../../utils/unit.converter';
import UIConstants from '../../utils/ui.constants';
import UtilService from '../../services/util.service';

const SchedulingConstraints = (props) => {
    let editorFunction = null;
    const { parentFunction = (editorFn) => { editorFunction = editorFn; } } = props;
    const [constraintSchema, setConstraintSchema] = useState();
    const [initialValue, setInitialValue] = useState();
    const [systemTime, setSystemTime] = useState();
    //SU Constraint Editor Property Order,format and validation
    const configureProperties = (properties) => {
        for (const propertyKey in properties) {
            const propertyValue = properties[propertyKey];
            if (propertyValue instanceof Object) {
                if (propertyKey !== "reference_pointing") {
                    configureProperties(propertyValue)
                }
                if (propertyKey === 'scheduler') {
                    propertyValue.propertyOrder = 1;
                }
                if (propertyKey === 'time') {
                    propertyValue.propertyOrder = 2;
                }
                if (propertyKey === 'at' || propertyKey === 'after' || propertyKey === 'before') {
                    propertyValue.propertyOrder = 3;
                }
                if (propertyKey === 'between' || propertyKey === 'not_between') {
                    propertyValue.propertyOrder = 4;
                }
                if (propertyKey === 'daily') {
                    propertyValue.propertyOrder = 5;
                    propertyValue.format = 'checkbox';
                    propertyValue.skipFormat = true;
                }
                if ((propertyKey === 'require_night' || propertyKey === 'require_day' || propertyKey === 'avoid_twilight') && typeof (propertyValue) !== "boolean") {
                    propertyValue.propertyOrder = 6;
                    propertyValue.format = 'checkbox';
                    propertyValue.skipFormat = true;
                }
                if (propertyKey === 'sky') {
                    propertyValue.propertyOrder = 7;
                }
                if (propertyKey === 'calibrator' || propertyKey === 'target') {
                    if (props.onUpdate !== "scheduleConstraints") {
                        propertyValue.default = ((propertyValue.default * 180) / Math.PI).toFixed(2);
                    }
                    propertyValue.title = propertyKey + ' (Degrees)';
                    propertyValue.propertyOrder = 8;
                    propertyValue.validationType = 'elevation';
                }
                if (propertyKey === 'transit_offset') {
                    propertyValue.propertyOrder = 9;
                }
                if (propertyKey === 'from') {
                    propertyValue.propertyOrder = 10;
                    propertyValue.validationType = 'transitOffset';
                }
                if (propertyKey === 'to') {
                    propertyValue.propertyOrder = 11;
                    propertyValue.validationType = 'transitOffset';
                }
                if (propertyKey === 'sun' || propertyKey === 'moon' || propertyKey === 'jupiter') {
                    propertyValue.default = ((propertyValue.default * 180) / Math.PI).toFixed(2);
                    propertyValue.title = propertyKey + ' (Degrees)';
                    propertyValue.propertyOrder = 12;
                    propertyValue.validationType = 'distanceOnSky';
                }
            }
        }
    };

    //Configuring Schema Definitions
    const configureDefinitions = (schema) => {
        for (const definitionName in schema.definitions) {
            if (definitionName === 'timestamp') {
                schema.definitions.timestamp = {
                    validationType: 'datetime',
                    type: schema.definitions.timestamp.type
                };
            }  else if (definitionName !== 'timewindow' && definitionName !== 'timestamp' &&  definitionName !== 'pointing') {
                schema.definitions[definitionName] = {
                    type: schema.definitions[definitionName].type
                };
            } else if (definitionName === 'timewindow') {
                for (let property in schema.definitions.timewindow.properties) {
                    if (property === 'to' || property === 'from') {
                        if (property === 'from') {
                            schema.definitions.timewindow.properties[property].propertyOrder = 1;
                        } else {
                            schema.definitions.timewindow.properties[property].propertyOrder = 2;
                            schema.definitions.timewindow.properties[property].title = 'until';
                        }
                    }
                }
            }
        }
    }

    // Callback function to pass the constraints JSON output to parent component
    const onEditForm = (jsonOutput, errors, ref) => {
        if (props.callback) {
            props.callback(jsonOutput, errors);
        }
    }

    const constraintStrategy = async () => {
        const currentSystemTime = moment.utc(await UtilService.getUTC())
        setSystemTime(currentSystemTime);
        const constraintTemplate = _.cloneDeep(props.constraintTemplate);
        if (constraintTemplate.ref_resolved_schema) {
            configureProperties(constraintTemplate.ref_resolved_schema.properties);
            configureDefinitions(constraintTemplate.ref_resolved_schema);
        }
        setConstraintSchema(constraintTemplate);
    };

    const modifyInitiValue = () => {
        const initValue = _.cloneDeep(props.initValue);
        // For DateTime
        for (let key in initValue.time) {
            if (typeof initValue.time[key] === 'string') {
                initValue.time[key] = moment(new Date((initValue.time[key] || '').replace('Z', ''))).format(UIConstants.CALENDAR_DATETIME_FORMAT);
            } else {
                initValue.time[key].forEach(time => {
                    for (let subKey in time) {
                        time[subKey] = moment(new Date((time[subKey] || '').replace('Z', ''))).format(UIConstants.CALENDAR_DATETIME_FORMAT);
                    }
                    return true;
                })
            }
        }
        if (initValue.time && !initValue.time.at) {
            initValue.time.at = '';
        }
        if (initValue.time && !initValue.time.after) {
            initValue.time.after = '';
        }
        if (initValue.time && !initValue.time.before) {
            initValue.time.before = '';
        }


        UnitConversion.radiansToDegree(initValue.sky);
        setInitialValue(initValue);
    }
    let jeditor = null;
    useEffect(() => {
        if (!props.constraintTemplate) {
            return;
        }
        if (constraintSchema && props.constraintTemplate.id !== constraintSchema.id) {
            jeditor = null;
        }
        UtilService.getUTC().then(utcTime => {
            setSystemTime(moment.utc(utcTime));
        });
        if (props.initValue) {
            modifyInitiValue();
        }
        constraintStrategy();
        if (editorFunction) {
            editorFunction();
        }
    }, [props.constraintTemplate, props.initValue]);
    if (constraintSchema && !jeditor) {
        jeditor = React.createElement(Jeditor, {
            id: "constraint_editor",
            title: "Scheduling Constraints specification",
            schema: constraintSchema.ref_resolved_schema,
            callback: onEditForm,
            initValue: initialValue,
            disabled: props.disable,
            formatOutput: props.formatOutput,
            parentFunction: parentFunction,
            defintionFormatter: configureDefinitions,
            resolveExtRef: false,
            systemTime: systemTime,
            showInGrid: false
        });
    }

    return (
        <>
            {constraintSchema ? jeditor : ""}
        </>
    );
};

export default SchedulingConstraints;