import UIConstants from "../../../utils/ui.constants";
import ScheduleService from "../../../services/schedule.service";
import ReservationService from "../../../services/reservation.service";
import AuthUtil from "../../../utils/auth.util";
import UtilService from "../../../services/util.service";
import moment from "moment/moment";
import TaskService from "../../../services/task.service";
import { getSunTimingItems } from "../helpers/week.view.helper";
import TimelineCommonUtils from "../common.utils";
import { getStore } from "../../../services/store.helper";

export async function fetchTimelineData(startTime, endTime, setData) {
    const from = startTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);
    const until = endTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);

    const schedulingUnits = await ScheduleService.getTimelineSlimBlueprints(from, until);
    const reservations = await fetchReservations(from, until)

    setData({
        schedulingUnits: schedulingUnits,
        reservations: reservations
    })
}

export async function fetchReservations(formattedStartTime, formattedEndTime) {
    let reservations = []
    const shouldFetchReservations = getStore(UIConstants.STORE_KEY_TIMELINE).reservationsToggle;
    if (shouldFetchReservations) {
        reservations = await ReservationService.getTimelineReservations(formattedStartTime, formattedEndTime);
    }
    return reservations
}

export async function fetchUserPermissions(setPermissions) {
    const permission = await AuthUtil.getUserRolePermission();
    setPermissions({
        weekView: permission.userRolePermission.weekoverview,
        schedulingUnit: permission.userRolePermission.scheduleunit
    })
}

export async function fetchSchedulerStatuses(setSchedulerSettings) {
    let isDynamicSchedulerOn, isFixedTimeSchedulerOn, isDynamicSchedulerActive
    await UtilService.getDynamicSchedulerStatus()
        .then(response => isDynamicSchedulerOn = response ? response.value : false)
    await UtilService.getFixedTimeSchedulerStatus()
        .then(response => isFixedTimeSchedulerOn = response ? response.value : false)
    await UtilService.getSubSystemScheduler()
        .then(response => isDynamicSchedulerActive = response ? response.status_value === "active" : false)
    setSchedulerSettings({
        isDynamicSchedulerOn: isDynamicSchedulerOn,
        isFixedTimeSchedulerOn: isFixedTimeSchedulerOn,
        isDynamicSchedulerActive: isDynamicSchedulerActive
    })
}

// we need to add sunrise and sunsite to all group items.
export async function fetchSunTimings(startTime, setItems, groups, groupkey) {
    if (groups.length == 0) return;
    const sunTimingItems = await getSunTimingItems(startTime, groups, groupkey)
    setItems(prevState => ({
        ...prevState,
        sunTimingsItems: sunTimingItems
    }))
}

export async function fetchLSTStationShift(utcTime, station) {
    let UTCTimeNow = moment(utcTime).format(UIConstants.UTC_DATE_TIME_FORMAT);
    let stations = [UIConstants.TIMINGSERVER]
    if (station) {
        if (station !== UIConstants.TIMINGSERVER) {
            stations = [station, UIConstants.TIMINGSERVER]
        }
        else {
            station = UIConstants.TIMINGSERVER;
            stations = [UIConstants.TIMINGSERVER]
        }
    }
    let lstresponse = await UtilService.getLST(UTCTimeNow, stations) // we get timing also, makes debugging a lot easier.
    if (!lstresponse) return -1;

    let lststation = lstresponse.data?.LST?.[station];
    let UTCAlsoTimeNow = moment.utc(lstresponse.data?.UTC).format(UIConstants.UTC_DATE_TIME_FORMAT);
    let lstcore = lstresponse.data?.LST?.[UIConstants.TIMINGSERVER];


    const shiftLSTStation = GetLstShiftinSeconds(lststation, UTCAlsoTimeNow)
    const shiftLSTCore = GetLstShiftinSeconds(lstcore, UTCAlsoTimeNow)

    return { shiftLSTStation, shiftLSTCore }
}

export async function fetchLSTShift(startTime, setHeaderSettings) {

    const shift = await fetchLSTStationShift(startTime, UIConstants.TIMINGSERVER);

    setHeaderSettings(prevState => ({
        ...prevState,
        lstShiftInSeconds: shift.shiftLSTCore
    }))

}

export function GetLstShiftinSeconds(lst, UTCTimeNow) {
    const LSTTimeNow = moment(lst, "HH:mm:ss");
    const timedifference = LSTTimeNow.diff(UTCTimeNow);
    const shift = moment.duration(timedifference).asSeconds();
    return shift;
}


// we get the missing_stations and the stations from the already present in memory information from the timeline view
// The Deeper subtasks information is removed.
async function addNestedBlueprintInformation(suBluePrint, inMemoryItem) {
    let missingStations = inMemoryItem.missing_stations;
    suBluePrint.stations = inMemoryItem.specified_stations;
    suBluePrint.missingStations = missingStations
    suBluePrint.project = inMemoryItem.project;
}

export async function fetchSUSummaryInformation(id, setSummarySettings, inMemoryItem) {
    let suBluePrint = await ScheduleService.getBlueprintByschedulingUnitId(id, "task_blueprints,draft")
    await addNestedBlueprintInformation(suBluePrint, inMemoryItem);

    const suConstraintTemplate = await ScheduleService.getSchedulingConstraintTemplateById(suBluePrint.scheduling_constraints_template_id)

    setSummarySettings(prevState => ({
        ...prevState,
        isLoading: false,
        schedulingUnitItem: suBluePrint,
        summaryConstraintTemplate: suConstraintTemplate
    }))

}

export async function fetchReservationSummaryInformation(id, setSummarySettings) {
    let reservation = await ReservationService.getReservation(id)
    setSummarySettings(prevState => ({
        ...prevState,
        isLoading: false,
        reservationItem: reservation,
    }))
}