import {
    getReservationItem,
    getReservationTypeColorIndex,
    getTimelineItem,
    splitObjectIfSpanIsMultipleDays,
    getStationsInfo
} from "./filters.helper";
import moment from "moment/moment";
import UIConstants from "../../../../utils/ui.constants";

jest.mock('moment', () => {
    const originalMoment = jest.requireActual('moment');
    return (date) => originalMoment(date);
});

describe("splitObjectIfSpanIsMultipleDays", () => {
    it("reservation times: splits an element that spans multiple days into multiple elements", () => {
        const element = {
            start_time: "2023-09-01T11:00:00",
            stop_time: "2023-09-03T10:00:00",
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element, false);

        expect(newElements).toHaveLength(3);

        //day 1: actual start to end of day
        expect(newElements[0].start_time, 'day 1 start incorrect').toBe("2023-09-01T11:00:00")
        expect(newElements[0].stop_time, 'day 1 stop incorrect').toBe("2023-09-01T23:59:59")

        //day 2: start of day to end of day
        expect(newElements[1].start_time, 'day 2 start incorrect').toBe("2023-09-02T00:00:00")
        expect(newElements[1].stop_time, 'day 2 stop incorrect').toBe("2023-09-02T23:59:59")

        //day 3: start of day to actual stop
        expect(newElements[2].start_time, 'day 3 start incorrect').toBe("2023-09-03T00:00:00")
        expect(newElements[2].stop_time, 'day 3 stop incorrect').toBe("2023-09-03T10:00:00")
    });

    it("process times: splits an element that spans multiple days into multiple elements", () => {
        const element = {
            process_start_time: "2023-09-01T11:00:00",
            process_stop_time: "2023-09-03T10:00:00",
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element, false);

        expect(newElements).toHaveLength(3);

        //day 1: actual start to end of day
        expect(newElements[0].process_start_time, 'day 1 start incorrect').toBe("2023-09-01T11:00:00")
        expect(newElements[0].process_stop_time, 'day 1 stop incorrect').toBe("2023-09-01T23:59:59")

        //day 2: start of day to end of day
        expect(newElements[1].process_start_time, 'day 2 start incorrect').toBe("2023-09-02T00:00:00")
        expect(newElements[1].process_stop_time, 'day 2 stop incorrect').toBe("2023-09-02T23:59:59")

        //day 3: start of day to actual stop
        expect(newElements[2].process_start_time, 'day 3 start incorrect').toBe("2023-09-03T00:00:00")
        expect(newElements[2].process_stop_time, 'day 3 stop incorrect').toBe("2023-09-03T10:00:00")
    });

    it("on sky times: splits an element that spans multiple days into multiple elements", () => {
        const element = {
            on_sky_start_time: "2023-09-01T11:00:00",
            on_sky_stop_time: "2023-09-03T10:00:00",
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element, true);

        expect(newElements).toHaveLength(3);

        //day 1: actual start to end of day
        expect(newElements[0].on_sky_start_time, 'day 1 start incorrect').toBe("2023-09-01T11:00:00")
        expect(newElements[0].on_sky_stop_time, 'day 1 stop incorrect').toBe("2023-09-01T23:59:59")

        //day 2: start of day to end of day
        expect(newElements[1].on_sky_start_time, 'day 2 start incorrect').toBe("2023-09-02T00:00:00")
        expect(newElements[1].on_sky_stop_time, 'day 2 stop incorrect').toBe("2023-09-02T23:59:59")

        //day 3: start of day to actual stop
        expect(newElements[2].on_sky_start_time, 'day 3 start incorrect').toBe("2023-09-03T00:00:00")
        expect(newElements[2].on_sky_stop_time, 'day 3 stop incorrect').toBe("2023-09-03T10:00:00")
    });

    it("Using the process start time if the on_sky start time is null", () => {
        const element = {
            process_start_time: "2023-09-01T11:00:00",
            process_stop_time: "2023-09-03T10:00:00",
            on_sky_start_time: null,
            on_sky_stop_time: null,
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element, true);

        expect(newElements).toHaveLength(3);

        //day 1: actual start to end of day
        expect(newElements[0].on_sky_start_time, 'day 1 start incorrect').toBe("2023-09-01T11:00:00")
        expect(newElements[0].on_sky_stop_time, 'day 1 stop incorrect').toBe("2023-09-01T23:59:59")

        //day 2: start of day to end of day
        expect(newElements[1].on_sky_start_time, 'day 2 start incorrect').toBe("2023-09-02T00:00:00")
        expect(newElements[1].on_sky_stop_time, 'day 2 stop incorrect').toBe("2023-09-02T23:59:59")

        //day 3: start of day to actual stop
        expect(newElements[2].on_sky_start_time, 'day 3 start incorrect').toBe("2023-09-03T00:00:00")
        expect(newElements[2].on_sky_stop_time, 'day 3 stop incorrect').toBe("2023-09-03T10:00:00")
    });

    it("splits an element that has no end time in multiple elements", () => {
        const element = {
            process_start_time: "2023-09-01T11:00:00",
            process_stop_time: null,
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element);

        expect(newElements).toHaveLength(8);

        //day 1: actual start to end of day
        expect(newElements[0].process_start_time, 'day 1 start incorrect').toBe("2023-09-01T11:00:00")
        expect(newElements[0].process_stop_time, 'day 1 stop incorrect').toBe("2023-09-01T23:59:59")

        //day 2: start of day to end of day
        expect(newElements[1].process_start_time, 'day 2 start incorrect').toBe("2023-09-02T00:00:00")
        expect(newElements[1].process_stop_time, 'day 2 stop incorrect').toBe("2023-09-02T23:59:59")
    });

    it("does not split an element that spans a single day", () => {
        const element = {
            start_time: "2023-09-01T12:00:00",
            stop_time: "2023-09-01T14:00:00",
        };
        const newElements = splitObjectIfSpanIsMultipleDays(element);

        expect(newElements).toHaveLength(1);
        expect(newElements[0].start_time).toEqual(element.start_time);
        expect(newElements[0].stop_time).toEqual(element.stop_time);
    });

    test.each([undefined, null])('Throws reference error when start and stop times are empty: %s', (emptyValue) => {
        const element = {
            start_time: emptyValue,
            stop_time: emptyValue,
        };
        expect(() => {
            splitObjectIfSpanIsMultipleDays(element);
        }).toThrow(ReferenceError);
    })

    test.each(['23-02-01', 'invalid'])('Throws type error when start and stop times are invalid: %s', (invalidValue) => {
        const element = {
            start_time: invalidValue,
            stop_time: invalidValue,
        };
        expect(() => {
            splitObjectIfSpanIsMultipleDays(element);
        }).toThrow(TypeError);
    })
});



describe("getTimelineItem", () => {
    const tasks = [
        {
            specifications_template: {
                type_value: "observation"
            },
            specifications_doc: {
                station_configuration: {
                    antenna_set: "antenna_set_value"
                }
            }
        }
    ]

    const suBlueprint = {
        id: "blueprint-id",
        description: "test blueprint",
        process_start_time: "2023-07-06T10:00:00Z",
        on_sky_start_time: "2023-07-06T10:00:00Z",
        tasks: tasks,
        project: "bugs",
        name: "the_fix_1",
        status: "defined",
        duration: 3600,
        on_sky_duration: 7200,
        process_stop_time: "2023-07-06T12:00:00Z",
        on_sky_stop_time: "2023-07-06T12:00:00Z",
        real_start_time: "2023-07-06T12:01:00Z",
        real_end_time: "2023-07-06T12:02:00Z",
        placed: true,
        unschedulable_reason: "none",
        scheduling_constraints_doc: { scheduler: "fixed" },
        antenna_set: "antenna_set_value",
        scheduler: "fixed",
        priority_queue:"A"
    };

    const displayDate = moment("2023-07-06", "YYYY-MM-DD")

    it("Returns the correct item object", () => {
        const item = getTimelineItem(suBlueprint, displayDate);

        expect(item).toEqual({
            SkyTimesOnTimes: false,
            id: "blueprint-id-27: Jul 06 - Thu",
            suId: "blueprint-id",
            group: "27: Jul 06 - Thu",
            band: "antenna",
            band_group: "antenna#2023-07-06",
            desc: "test blueprint",
            title: "",
            project: "bugs",
            project_task_group: "bugs#2023-07-06#tasks",
            project_group: "bugs#2023-07-06",
            name: "the_fix_1",
            name_group:"the_fix_1#2023-07-06",
            antenna_set: "antenna_set_value",
            antenna_set_group: "antenna_set_value#2023-07-06",
            duration: "1.00Hrs",
            priority_queue: "A",
             priority_queue_group: "A#2023-07-06",
            on_sky_duration: "2.00Hrs",
            start_time: moment("2023-07-06 10:00:00Z"),
            end_time: moment("2023-07-06 12:00:00Z"),
            real_start_time: "2023-07-06T12:01:00Z",
            real_end_time: "2023-07-06T12:02:00Z",
            stations: "-",
            type: "SCHEDULE",
            status: "defined",
            scheduler: "fixed",
            scheduler_group: "fixed#2023-07-06",
            displayDate: moment("2023-07-06"),
            unschedulable_reason: "none"
        });
    });



    it("returns the correct item object when antenna set is not provided", () => {
        let suBlueprintWithoutAntennaSet = {
            ...suBlueprint,
            tasks: [
                {
                    specifications_template: {
                        type_value: "observation"
                    },
                    specifications_doc: {}
                }
            ]
        };
        suBlueprintWithoutAntennaSet.antenna_set = undefined;
        const item = getTimelineItem(suBlueprintWithoutAntennaSet, displayDate);

        expect(item).toEqual({
            SkyTimesOnTimes: false,
            id: "blueprint-id-27: Jul 06 - Thu",
            suId: "blueprint-id",
            group: "27: Jul 06 - Thu",
            band: undefined,
            desc: "test blueprint",
            title: "",
            project: "bugs",
            project_group: "bugs#2023-07-06",
            name: "the_fix_1",
            name_group:"the_fix_1#2023-07-06",
            antenna_set: undefined,
            duration: "1.00Hrs",
            priority_queue: "A",
             priority_queue_group: "A#2023-07-06",
             project_task_group: "bugs#2023-07-06#tasks",
            on_sky_duration: "2.00Hrs",
            start_time: moment("2023-07-06 10:00:00Z"),
            end_time: moment("2023-07-06 12:00:00Z"),
            real_start_time: "2023-07-06T12:01:00Z",
            real_end_time: "2023-07-06T12:02:00Z",
            stations: "-",
            type: "SCHEDULE",
            status: "defined",
            scheduler: "fixed",
            scheduler_group: "fixed#2023-07-06",
            displayDate: moment("2023-07-06"),
            unschedulable_reason: "none"
        });
    });
});

describe('getReservationItem', () => {


    let reservation = {
        id: '1',
        name: 'green grass',
        specifications_doc: {
            activity: {
                type: 'mowing',
                planned: 'yes'
            },
            schedulability: {
                fixed_time: true,
                dynamic: false
            }
        },
        start_time: '2023-08-08 10:00:00',
        stop_time: '2023-08-08 12:00:00',
        duration: 7200,
        project_id: '123',
        description: 'with a grass mower'
    };


    let expectedItem = {
        id: "1~0",
        start_time: moment('2023-08-08 10:00:00'),
        real_start_time: "2023-08-08 10:00:00",
        end_time: moment('2023-08-08 12:00:00'),
        real_end_time: "2023-08-08 12:00:00",
        group: "32: Aug 08 - Tue",
        groupingSuffix: "#2023-08-08",
        project_task_group: "123#2023-08-08",
        project_group: "123#2023-08-08",
        name: 'green grass',
        name_group: "green grass#2023-08-08",
        project: '123',
        type: 'RESERVATION',
        activity_type: 'mowing - 123',
        title: '',
        desc: 'with a grass mower',
        duration: '02:00:00',
        bgColor: "#585859",
        selectedBgColor: "#585859",
        color: 'white',
        stations: '-',
        reservationId:'1',
        reason:'mowing',
        planned: 'yes',
        specified_stations: undefined,
        reservation_start_time: moment('2023-08-08 10:00:00'),
        stationsgroups:  {
                 CS: 0,
                 IS: 0,
                 RS: 0,
               },
    };


    const displayDate = moment('2023-08-08', 'YYYY-MM-DD').format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT);

    it('should return the correct reservation item (no stations)', () => {
        const result = getReservationItem(reservation, 0, displayDate);
        expect(result).toEqual(expectedItem);
    });




    it('should return the correct reservation item (with stations)', () => {
        reservation.specifications_doc.resources = { stations: ['CS001', 'CS002' ,'CS003'] }

        expectedItem.stations = 'CS001,CS002,CS003';
        expectedItem.specified_stations =  [
                 "CS001",
                 "CS002",
                 "CS003",
               ];

               expectedItem.stationsgroups=  {
                CS: 3,
                IS: 0,
                RS: 0,
              };

        const result = getReservationItem(reservation, 0, displayDate);

        expect(result).toEqual(expectedItem);
    });

});

describe('getReservationTypeColorIndex', () => {
    it('should return the correct color for fixed time and dynamic schedulability', () => {
        const schedulability = { fixed_time: true, dynamic: true };
        const expectedColor = { bgColor: 'lightgrey', color: '#585859' };

        const result = getReservationTypeColorIndex(schedulability);

        expect(result).toEqual(expectedColor);
    });

    it('should return the correct color for fixed time and not dynamic schedulability', () => {
        const schedulability = { fixed_time: true, dynamic: false };
        const expectedColor = { bgColor: '#585859', color: "white" };

        const result = getReservationTypeColorIndex(schedulability);

        expect(result).toEqual(expectedColor);
    });

    it('should return the correct color for not fixed time and dynamic schedulability', () => {
        const schedulability = { fixed_time: false, dynamic: true };
        const expectedColor = { bgColor: "#9b9999", color: "white" };

        const result = getReservationTypeColorIndex(schedulability);

        expect(result).toEqual(expectedColor);
    });

    it('should return the correct color for not fixed time and not dynamic schedulability', () => {
        const schedulability = { fixed_time: false, dynamic: false };
        const expectedColor = { bgColor: '#212121', color: 'white' };

        const result = getReservationTypeColorIndex(schedulability);

        expect(result).toEqual(expectedColor);
    });

    it('should return the correct color for undefined schedulability', () => {
        const schedulability = {};
        const expectedColor = { bgColor: '#212121', color: 'white' };

        const result = getReservationTypeColorIndex(schedulability);

        expect(result).toEqual(expectedColor);
    });
});

describe('getStationsInfo', ()=> {
    it('should Give Correctly the stations for 3 items', () => {
      const stations =  getStationsInfo(['CS001','CS002','CS003']);
      expect (stations.summary).toEqual("CS001,CS002,CS003");
    });
    it('should Give Correctly the stations for no items', () => {
        const stations =  getStationsInfo([]);
        expect (stations.summary).toEqual("-");
      });


      it('should Count Correctly the Core stations', () => {
        const stations =  getStationsInfo(['CS001','CS002','CS003','CS021']);
        expect (stations.summary).toEqual("CS/RS/IS : 4/0/0");
      }); 

      it('should Count Correctly the RS  stations', () => {
        const stations =  getStationsInfo(['RS001','RS002','RS003','RS021','RS026']);
        expect (stations.summary).toEqual("CS/RS/IS : 0/5/0");
      }); 

      it('should Count Correctly the IS  stations', () => {
        const stations =  getStationsInfo(['DE001','IT002','FR003','UK021','UK023','IC024']);
        expect (stations.summary).toEqual("CS/RS/IS : 0/0/6");
      }); 

      it('should Count Correctly Combined  stations', () => {
        const stations =  getStationsInfo(['CS001','CS002','RS001','RS002','DE001','IT002','FR003','UK021','UK023','RS003','RS021','RS026','CS003','IC024','CS021']);
        expect (stations.summary).toEqual("CS/RS/IS : 4/5/6");
      }); 



});
