import axios from "axios"
import {parseTemplatesToCorrectJSONFormat} from "./service.helper";

const TaskService = {
    getTaskDetails: async function (taskType, taskId, fetchSubtask) {
      try {
        const responseData = await this.getTask(taskType, taskId, fetchSubtask);
        responseData.predecessors = [];
        responseData.successors = [];
        if (taskType === 'blueprint') {
          responseData.blueprints = [];
        } else {
          responseData.draftName = null;
        }
        return this.getTaskRelationsByTask(taskType, responseData)
            .then(relations => {
            responseData.predecessors = relations.predecessors;
            responseData.successors = relations.successors;
            if (taskType === 'draft') {
              responseData.blueprints = relations.blueprints;
            } else {
              responseData.draftObject = relations.draft;
            }
            return responseData;
        });

      } catch (error) {
        console.error(error);
      }
    },
    getTask : async function (taskType, taskId, fetchSubtask) {
      try {
        let url = ''
        if(fetchSubtask){
          url = taskType === 'blueprint'? '/api/task_blueprint/'+taskId+'?expand=subtasks': '/api/task_draft/'+taskId;
        }
        else {
          url = taskType === 'blueprint'? '/api/task_blueprint/'+taskId: '/api/task_draft/'+taskId;
        }
        const response = await axios.get(url);// + taskId);
        return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    getTaskTemplate: async function(templateId, refResolveSchema=false) {
      try {
        const response = await axios.get(`/api/task_template/${templateId}${refResolveSchema ? '/ref_resolved_schema' : ''}`);
        return response.data;
      } catch (error) {
        console.log(error);
      }
    },
    getTaskTemplates: async function() {
      let templates = [];
      try {
        let url = '/api/task_template/'
        let response = await axios.get(url);
        const totalCount = response.data.count;
        const initialCount = response.data.results.length;
        templates = response.data.results;
        if (totalCount > initialCount) {
            url = `${url}?limit=${totalCount-initialCount}&offset=${initialCount}`;
            response = await axios.get(url);
            templates = templates.concat(response.data.results);
        }
      } catch (error) {
        console.log(error);
      }
      return parseTemplatesToCorrectJSONFormat(templates);
    },
    getTaskTypes: async() => {
        try {
            const response = await axios.get('/api/task_type/');
            return response.data.results;
        } catch (error) {
            console.log(error);
        }
    },
    getSchedulingUnit: async function(type, id, loadTasks, fields) {
      try {
        let url = `/api/scheduling_unit_${type}/${id}`;
        if (loadTasks) {
          url += `/?expand=task_${type}s,task_${type}s.specifications_template`;
        }
        url += (fields && fields !== '')? '&fields='+fields : '' ;
        const response = await axios.get(url);
        return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    updateTask: async function(type, task) {
      try {
        const response = await axios.put(('/api/task_draft/' + task.id + "/"), task);
        return response.data;
      } catch (error) {
        console.error(error);
        return {
          error: true,
          message: 'Unable to update task'
        };
      }
    },
    getTaskRelationsByTask: async function(type, task) {
      try {
        let relations = {};
        return this.getTaskPredecessors(type, task.id, task.task_type)
        .then( predecessors => {
          relations.predecessors = predecessors;
          return this.getTaskSuccessors(type, task.id);
        })
        .then( successors => {
          relations.successors = successors;
          if (type === 'draft') {
            return this.getDraftsTaskBlueprints(task.id, 'id,name');
          } else {
            return this.getTaskdraft(task.draft_id, 'id,name');
          }
        })
        .then( result => {
          if (type === 'draft') {
            relations.blueprints = result;
          } else {
            relations.draft = result;
          }
          return relations;
        });
      } catch (error) {
        console.log(error);
      }
    },
    getTaskPredecessors: async function(type, id, taskType) {
      try {
        let url = type === 'blueprint'? `/api/task_blueprint/${id}/predecessors`: `/api/task_draft/${id}/predecessors/`;
        if (taskType === 'pipeline') {
          url += '?expand=specifications_template'
        }
        const response = await axios.get(url);
        return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    getTaskSuccessors: async function(type, id) {
      try {
        const url = type === 'blueprint'? `/api/task_blueprint/${id}/successors/?expand=subtasks`: `/api/task_draft/${id}/successors/`;
        const response = await axios.get(url);
        return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    getDraftsTaskBlueprints: async function(id, fields) {
      try {
        let api = `/api/task_draft/${id}/task_blueprint/?`;
        api += (fields && fields !== '')? 'fields='+fields : '' ;
        const response = await axios.get(api);
        return response.data.results;
      } catch (error) {
        console.error(error);
      }
    },
    getTaskdraft: async function(id, fields) {
      try {
        let api = `/api/task_draft/${id}/?`;
        api += (fields && fields !== '')? 'fields='+fields : '' ;
        const response = await axios.get(api);
        return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    validateSpecificationsDoc: async function(task, specifications_doc=null) {
      try {
            const url = `${task.specifications_template}/validate_specifications_doc`;
            const response = await axios.put(url, {'specifications_doc': specifications_doc? specifications_doc : task.specifications_doc} );
            if(!response.data['valid']) {
                console.error(response.data['message']);
            }

            return response.data;
      } catch (error) {
        console.error(error);
      }
    },
    getSubtaskStatusLogs: async function(subtaskId) {
      try {
        const response = await axios.get(`/api/subtask/${subtaskId}/state_log`);
        return response.data;
      } catch(error) {
        console.error(error);
      }
    },
    getTaskStatusLogs: async function(taskId) {
      let statusLogs = [];
      try {
        let subtaskTemplates = {};
        const taskDetails = (await axios.get(`/api/task_blueprint/${taskId}`)).data;
        for (const subtaskId of taskDetails.subtasks_ids) {
          const subtaskDetails = await this.getSubtaskDetails(subtaskId);
          const subtaskLogs = await this.getSubtaskStatusLogs(subtaskId);
          let template = subtaskTemplates[subtaskDetails.specifications_template_id];
          if (!template) {
            template = (await this.getSubtaskTemplate(subtaskDetails.specifications_template_id));
            subtaskTemplates[subtaskDetails.specifications_template_id] = template;
          }
          for (let statusLog of subtaskLogs) {
            statusLog.subtask_type = template?template.name:"";
          }
          statusLogs = statusLogs.concat(subtaskLogs);
        }
      } catch(error) {
        console.error(error);
      }
      return statusLogs;
    },
    getSubtaskTemplates: async function() {
      let templates = [];
      try {
        let url = '/api/subtask_template/'
        let response = await axios.get(url);
        const totalCount = response.data.count;
        const initialCount = response.data.results.length;
        templates = response.data.results;
        if (totalCount > initialCount) {
            url = `${url}?limit=${totalCount-initialCount}&offset=${initialCount}`;
            response = await axios.get(url);
            templates = templates.concat(response.data.results);
        }
      } catch (error) {
        console.log(error);
      }
      return templates;
    },
    getSubtaskTemplate: async function(templateId) {
      try {
        const response = await axios.get(`/api/subtask_template/${templateId}`);
        return response.data;
      } catch(error) {
        console.error(error);
      }
    },
    getSubtaskDetails: async function(subtaskId, fields= undefined) {
      try {
        return (await axios.get("/api/subtask/" + subtaskId + (fields ? "?fields=" + fields : ""))).data;
      } catch(error) {
        console.error(error);
      }
    },
    getSubtaskMissingStations: async function(subtaskId) {
      try {
        return (await axios.get(`/api/subtask/${subtaskId}/missing_stations`)).data['missing_stations'];
      } catch(error) {
        console.error(error);
      }
    },
    getSubtaskType: async function() {
      try {
        return (await axios.get(`/api/subtask_type`)).data.results;

      } catch(error) { /* empty */ }
    },
    /**
     * Delete task based on task type
     * @param {*} type
     * @param {*} id
     */
    deleteTask: async function(type, id) {
        try {
            const url = type.toLowerCase() === 'blueprint'? `/api/task_blueprint/${id}`: `/api/task_draft/${id}`;
            await axios.delete(url);
            return true;
        } catch(error) {
            console.error(error);
            return false;
        }
    },
    /**
     * Cancel task
     * @param {*} id
     */
    cancelTask: async function(id) {
        try {
            const url = `/api/task_blueprint/${id}/cancel`;
            const response = await axios.post(url, {});
            return response.data;
        } catch(error) {
            console.error(error);
            return false;
        }
    },
    cancelSubtask: async function(id) {
        try {
            const url = `/api/subtask/${id}/cancel/`;
            const res = await axios.post(url, {});
            return res.data;
        } catch(error) {
            console.error(error);
            return null;
        }
    },
    markSubtaskObsolete: async function(id) {
        try {
            const url = `/api/subtask/${id}/mark_as_obsolete/`;
            const res = await axios.post(url, {});
            return res.data;
        } catch(error) {
            console.error(error);
            return null;
        }
    },
    markTaskObsolete: async function(id) {
        try {
            const url = `/api/task_blueprint/${id}/mark_as_obsolete/`;
            const res = await axios.post(url, {});
            return res.data;
        } catch(error) {
            console.error(error);
            return null;
        }
    },
    getTaskFilterDefinition: async function (type){
        let res = [];
        try {
            res = await axios.options(`/api/task_${type}/`);
        }   catch(error) {
            console.error('[Task.services.getTaskFilterDefinition]',error);
        }
        return res;
    },
    getExpandedTasksWithFilter: async function (type, expand, fields, filters, orderBy, limit, offset){
        let response = null;
        try {
            let api = `/api/task_${type}/?`;
            api += (expand === '')? '' : 'expand='+expand+'&';
            api += (fields && fields !== '')? 'fields='+fields+'&' : '' ;
            api += (filters === '')? '' : filters+'&';
            api += (orderBy === '')? '' : orderBy+'&';
            api += 'limit='+limit+'&offset='
            api += (offset === '') ? 0 : offset;
            response = await axios.get(api);
        }   catch(error) {
            console.error('[schedule.services.getExpandedTasksWithFilter]',error);
        }
        return response;
    },
    getTaskCount: async function(type) {
      try {
          return (await axios.get(`/api/task_${type}/?fields=id&limit=1`)).data.count
      } catch(error) {
        console.error(error);
        return 0;
      }
    },
    updateTaskValue: async function(task) {
      try {
        let url = `/api/task_draft_extended/${task.id}/`;
        const response = await axios.patch(url, task);
        return response.data;
      } catch (error) {
          console.error('[task.services.updateTaskValue]',"Error", error);
          return null
      }
    },
    restartTaskOnHold: async function(taskId) {
      try {
        let url = `/api/task_blueprint/${taskId}/restart_when_on_hold`;
          return await axios.post(url);
      } catch (error) {
          console.error('[task.services.restartTaskOnHold]',"Error", error);
          return null
      }
    },
    restartSubtaskTaskOnHold: async function(subtaskId) {
      try {
        let url = `/api/subtask/${subtaskId}/restart_when_on_hold`;
          return await axios.post(url);
      } catch (error) {
          console.error('[task.services.restartSubtaskTaskOnHold]',"Error", error);
          return null
      }
    },
    getSubtaskProgress: async (subtaskId) => {
      try {
        let url = `/api/subtask/${subtaskId}/get_progress`;
        const response = await axios.get(url);
        return response.data;
      } catch (error) {
          console.error('[task.services.getSubtaskProgress]',"Error", error);
          return null
      }
    },
    /**
     * Cancel task
     * @param task
     */
     createTask: async function(task) {
      try {
          const url = `/api/task_draft`;
          return await axios.post(url, task);
      } catch(error) {
          console.error(error);
          return false;
      }
  }
}

export default TaskService;
