import _ from 'lodash';
import $RefParser from "@apidevtools/json-schema-ref-parser";

const ParserUtility = {
    /**
     * Function to get the property of the parameter referred for the task.
     * @param {*} $strategyRefs 
     * @param {Array} paramPaths - Property reference path. 
     * Example if the task parameter refers '#/tasks/Target Pointing 1/specifications_doc/SAPs/0/digital_pointing',
     * then the parameter to be passed is [digital_pointing, 0, SAPs]
     * @param {Object} taskTemplateSchema - JSON schema for the respective task template
     * @returns 
     */
     getParamProperty: async($strategyRefs, paramPaths, taskTemplateSchema, taskFilters) => {
        const $templateRefs = await $RefParser.resolve(taskTemplateSchema);
        let pathIndex = 0;
        let paramProp = {};
        for (const paramPath of paramPaths) {
            let property = taskTemplateSchema.properties[paramPath];
            if (!property) {
                if (taskFilters && taskFilters.data?.actions?.POST) {
                    property = taskFilters.data.actions.POST[paramPath];
                }   else {
                    property = {type: "string"};
                }
            }
            if (property) {
                let rootPath = paramPaths.slice(0, pathIndex)
                rootPath.reverse();
                paramProp = _.cloneDeep(property);
                if (rootPath.length > 0) {
                    for (const path of rootPath) {
                        if (paramProp[path]) {
                            break;
                        }   else {
                            if (paramProp['$ref']) {
                                paramProp = $templateRefs.get(paramProp['$ref']);
                                if (paramProp.properties && paramProp.properties[path]) {
                                    paramProp = paramProp.properties[path];
                                }
                            }   else {
                                if (paramProp.type === "array" && paramProp.items.properties && paramProp.items.properties[path]) {
                                    paramProp = paramProp.items.properties[path];
                                }   else if (paramProp.type === "array" && paramProp.items['$ref']) {
                                    paramProp = $templateRefs.get(paramProp.items['$ref']);
                                    if (paramProp.properties && paramProp.properties[path]) {
                                        paramProp = paramProp.properties[path];
                                    }
                                }   else if (paramProp.type === "object" && paramProp.properties && paramProp.properties[path]) {
                                    paramProp = paramProp.properties[path];
                                }   else {
                                    // paramProp = paramProp[path];
                                }
                            }
                        }
                    }
                }
            }
            pathIndex++;
        }
        return paramProp;
    },

    /**
     * To get the default value for parameters if there is no default defined based on the property type
     * @param {string} paramType 
     * @returns 
     */
    getDefaultParamValue: (paramType) => {
        switch(paramType) {
            case "string": {
                return "";
            }
            case "object": {
                return {};
            }
            case "array": {
                return [];
            }
            case "boolean": {
                return false;
            }
            case "integer": {
                return 0;
            }
            case "number": {
                return 0;
            }
            default: {
                return "";
            }
        }
    },
    /**
     * Inject Station parameter if not defined in the strategy parameter
     * @param {*} observStrategy 
     */
    addStationParameters(observStrategy) {
        let isUpdated = false;
        if (observStrategy.template ) {
            const parameters = observStrategy.template.parameters;
            let stationParameters = parameters.filter( parameter => _.endsWith(parameter['refs'],'station_groups'));
            if (stationParameters === undefined || stationParameters.length === 0) {
                const taskKeys = Object.keys(observStrategy.template.tasks);
                for (const key of taskKeys) {
                    if (observStrategy.template.tasks[key]['specifications_doc'].hasOwnProperty(['station_configuration']) 
                    && observStrategy.template.tasks[key]['specifications_doc']['station_configuration'].hasOwnProperty(['station_groups'])) {
                        observStrategy.template.parameters.push({
                            "name": "Station Groups - "+key,
                            "refs": [
                                `#/tasks/${key}/specifications_doc/station_configuration/station_groups`
                            ]
                        });
                        isUpdated = true;
                    }                
                }
            }
        }
        return isUpdated;
    },
    /**
     * Reorders the schema properties and collapse the last two properties if required
     * @param {Objec} schema - schema JSON object 
     * @param {boolean} collapsed - to collpse the last two properties or not 
     * @returns Object - updated schema
     */
    updateReservationPropsOrder(schema, collapsed) {
        const propOrders = ["resources", "schedulability", "activity", "effects"];
        for (const propOrder of propOrders) {
            const propIndex = propOrders.indexOf(propOrder);
            schema.properties[propOrder].propertyOrder =  propIndex + 1;
            if (propIndex > 1) {
                schema.properties[propOrder].options = {collapsed: collapsed};
            }
        }
        return schema;
    }
}

export default ParserUtility;