import _ from 'lodash';
import ScheduleService from "../services/schedule.service";
import ReservationService from "../services/reservation.service";
import useWebSocket from 'react-use-websocket';
import { getStore } from "../services/store.helper";
import UIConstants from "../utils/ui.constants";


const isDebugLoggingOn =false;
function useWeekViewWebSocket(data, setData, summaryItem, setSummaryItem, setShowSummary) {
    
    /**
     * Function to call when websocket is connected
     */
    function onConnect() {
        try {
            if (isDebugLoggingOn) console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                sendMessage(JSON.stringify({ "token": JSON.parse(userDets).websocket_token }));
                if (isDebugLoggingOn)  console.log("Auth token submitted");
            }
        } catch (err) {
            console.log('err', err);
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    function onDisconnect() {
        if (isDebugLoggingOn) console.log("WS Closed");
    }

    function fetchBlueprintAndAddToTimeline(id) {
        // The websocket message only contains a subset of the details we need, so fetch the full set
        ScheduleService.getTimelineSlimBlueprints(undefined, undefined, id)   // todo: check time
                       .then((response) => {
                           setData(prevData => ({
                               ...prevData,
                               schedulingUnits: prevData.schedulingUnits.concat(response)
                           }));
                       })
                       .catch(e => console.error("Couldn't retrieve scheduling unit details for id: ", id, e));
    }


    function fetchReservationAndAddToTimeline(id) {
        const shouldFetchReservations = getStore(UIConstants.STORE_KEY_TIMELINE).reservationsToggle;
        if (shouldFetchReservations) {
            ReservationService.getTimelineReservations(undefined, undefined, id)  // todo: check time
                              .then((response) => {
                                  setData(prevData => ({
                                  ...prevData,
                                  reservations: prevData.reservations.concat(response)
                                  }));
                              })
                              .catch(e => console.error("Couldn't retrieve reservation details for id: ", id, e));
        }
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    function handleData(event) {
        const jsonData = JSON.parse(event?.data) || {};
        switch (jsonData.object_type) {
            case 'scheduling_unit_blueprint': {
                switch (jsonData.action) {
                    case 'delete': {
                        const schedulingUnits = data.schedulingUnits
                        _.remove(schedulingUnits, function (su) { return su.id === jsonData.object_details.id });
                        setData(prevData => ({
                            ...prevData,
                            schedulingUnits: schedulingUnits
                        }))
                        if (summaryItem?.id === jsonData.object_details.id) {
                            setShowSummary(false);
                        }
                        break;
                    }
                    case 'update': {
                        if (data.schedulingUnits.some(unit => unit.id === jsonData.object_details.id)) {
                            // usually we already have most details, so only update the relevant ones.
                            setData(prevData => ({
                                ...prevData,
                                schedulingUnits: prevData.schedulingUnits.map(
                                    unit => unit.id === jsonData.object_details.id ? { ...unit, ...jsonData.object_details } : unit
                                )
                            }));
                        } else {
                            // ...but sometimes we don't have the details yet, e.g. because it at least used to be
                            // outside the timelines time range, and we need to fetch all details so we don't miss
                            // anything that e.g. got moved into scope.
                            fetchBlueprintAndAddToTimeline(jsonData.object_details.id);
                        }

                        if (summaryItem?.id === jsonData.object_details.id) {
                            // Note: we could also update details directly so we don't need to fetch again.
                            // However, we would have to send full deltas via websocket instead of the subset that is
                            // relevant for the timeline (like id, timestamps, and status).
                            // e.g.
                            //
                            // setSummarySettings(prevState => ({
                            //    ...prevState,
                            //    schedulingUnitItem: {...prevState.schedulingUnitItem, ...jsonData.object_details}
                            // }));
                            //
                            // Instead, trigger a full refresh of the details panel:
                            setSummaryItem({ id: jsonData.object_details.id, type: "SCHEDULE" });
                        }
                        break;
                    }
                    case 'create': {
                        // The websocket message only contains a subset of the details we need, so fetch the full set
                        fetchBlueprintAndAddToTimeline(jsonData.object_details.id);
                        break;
                    }
                    default: { break; }
                }
                break;
            }
            case 'reservation': {
                switch (jsonData.action) {
                    case 'delete': {
                        const reservations = data.reservations
                        _.remove(reservations, function (res) { return res.id === jsonData.object_details.id });
                        setData(prevData => ({
                            ...prevData,
                            reservations: reservations
                        }))
                        if (summaryItem?.id === jsonData.object_details.id) {
                            setShowSummary(false);
                        }
                        break;
                    }
                    case 'update': {
                        if (data.reservations.some(res => res.id === jsonData.object_details.id)) {
                            // usually we already have most details, so only update the relevant ones.
                            setData(prevData => ({
                                ...prevData,
                                reservations: prevData.reservations.map(
                                    res => res.id === jsonData.object_details.id ? { ...res, ...jsonData.object_details } : res
                                ),
                            }));
                        } else {
                            // ...but sometimes we don't have the details yet, e.g. because it at least used to be
                            // outside the timelines time range, and we need to fetch all details so we don't miss
                            // anything that e.g. got moved into scope.
                            fetchReservationAndAddToTimeline(jsonData.object_details.id);
                        }
                        if (summaryItem?.id === jsonData.object_details.id) {
                            // Trigger a full refresh of the details panel
                            setSummaryItem({ id: jsonData.object_details.id, type: "RESERVATION" });
                        }
                        break;
                    }
                    case 'create': {
                        // The websocket message only contains a subset of the details we need, so fetch the full set
                        fetchReservationAndAddToTimeline(jsonData.object_details.id);
                        break;
                    }
                    default: { break; }
                }
                break;
            }
            default: { break; }
        }
    }

    // websocket hook that opens and allows interaction via the wss connection
    const {
        sendMessage
    } = useWebSocket(process.env.REACT_APP_WEBSOCKET_URL, {
        onOpen: () => onConnect(),
        onClose: () => onDisconnect(),
        onMessage: (event) => handleData(event),
        onError: (event) => { console.error(event); },
        shouldReconnect: (closeEvent) => true,
    });

}

export default useWeekViewWebSocket;